-- The ex3 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/01/02

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_ex8 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  OutChSel : in std_logic_vector(7 downto 0);
  CO300 : in std_logic_vector(8 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO128 : in std_logic_vector(9 downto 0);
  CO128_dly2 : in std_logic_vector(9 downto 0);
  X_RD_DATA : in std_logic_vector(1023 downto 0); 
  ReluAll : out std_logic_vector(2047 downto 0); 
  ReluAllWrN : out std_logic; 
  ReluSum : out std_logic_vector(11 downto 0) ); 
end conv_ex8;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_ex8 is

component dotpro_ex8
   Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(7 downto 0);
  MatRomData : in std_logic_vector(1 downto 0);
  CO9is1 : in std_logic;
  CO128is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(16 downto 0) );
end component;

component ROM_EW8
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(511 downto 0) );
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal CO9_clr : std_logic;
signal W3BUSY, W3BUSY_dly1, W3BUSY_dly2, W3BUSY_dly3, W3BUSY_dly4 : std_logic;

signal mac_000 : std_logic_vector(16 downto 0);

signal truncate_000 : std_logic_vector(9 downto 0);

signal Relu_000 : std_logic_vector(7 downto 0);

signal PixRomData : std_logic_vector(7 downto 0);
signal EW8Addr, EW8Addr_prev : std_logic_vector(10 downto 0);
signal EW8Addr_L : std_logic_vector(3 downto 0);
signal EW8Addr_H : std_logic_vector(10 downto 0);
signal EW8Sel : std_logic_vector(7 downto 0);
signal OutChSel_dly1, OutChSel_dly2 : std_logic_vector(7 downto 0);

signal reg_000_000, reg_000_001, reg_000_002, reg_000_003, reg_000_004, reg_000_005, reg_000_006, reg_000_007, reg_000_008, reg_000_009 : std_logic_vector(7 downto 0);
signal reg_000_010, reg_000_011, reg_000_012, reg_000_013, reg_000_014, reg_000_015, reg_000_016, reg_000_017, reg_000_018, reg_000_019 : std_logic_vector(7 downto 0);
signal reg_000_020, reg_000_021, reg_000_022, reg_000_023, reg_000_024, reg_000_025, reg_000_026, reg_000_027, reg_000_028, reg_000_029 : std_logic_vector(7 downto 0);
signal reg_000_030, reg_000_031, reg_000_032, reg_000_033, reg_000_034, reg_000_035, reg_000_036, reg_000_037, reg_000_038, reg_000_039 : std_logic_vector(7 downto 0);
signal reg_000_040, reg_000_041, reg_000_042, reg_000_043, reg_000_044, reg_000_045, reg_000_046, reg_000_047, reg_000_048, reg_000_049 : std_logic_vector(7 downto 0);
signal reg_000_050, reg_000_051, reg_000_052, reg_000_053, reg_000_054, reg_000_055, reg_000_056, reg_000_057, reg_000_058, reg_000_059 : std_logic_vector(7 downto 0);
signal reg_000_060, reg_000_061, reg_000_062, reg_000_063, reg_000_064, reg_000_065, reg_000_066, reg_000_067, reg_000_068, reg_000_069 : std_logic_vector(7 downto 0);
signal reg_000_070, reg_000_071, reg_000_072, reg_000_073, reg_000_074, reg_000_075, reg_000_076, reg_000_077, reg_000_078, reg_000_079 : std_logic_vector(7 downto 0);
signal reg_000_080, reg_000_081, reg_000_082, reg_000_083, reg_000_084, reg_000_085, reg_000_086, reg_000_087, reg_000_088, reg_000_089 : std_logic_vector(7 downto 0);
signal reg_000_090, reg_000_091, reg_000_092, reg_000_093, reg_000_094, reg_000_095, reg_000_096, reg_000_097, reg_000_098, reg_000_099 : std_logic_vector(7 downto 0);
signal reg_000_100, reg_000_101, reg_000_102, reg_000_103, reg_000_104, reg_000_105, reg_000_106, reg_000_107, reg_000_108, reg_000_109 : std_logic_vector(7 downto 0);
signal reg_000_110, reg_000_111, reg_000_112, reg_000_113, reg_000_114, reg_000_115, reg_000_116, reg_000_117, reg_000_118, reg_000_119 : std_logic_vector(7 downto 0);
signal reg_000_120, reg_000_121, reg_000_122, reg_000_123, reg_000_124, reg_000_125, reg_000_126, reg_000_127, reg_000_128, reg_000_129 : std_logic_vector(7 downto 0);
signal reg_000_130, reg_000_131, reg_000_132, reg_000_133, reg_000_134, reg_000_135, reg_000_136, reg_000_137, reg_000_138, reg_000_139 : std_logic_vector(7 downto 0);
signal reg_000_140, reg_000_141, reg_000_142, reg_000_143, reg_000_144, reg_000_145, reg_000_146, reg_000_147, reg_000_148, reg_000_149 : std_logic_vector(7 downto 0);
signal reg_000_150, reg_000_151, reg_000_152, reg_000_153, reg_000_154, reg_000_155, reg_000_156, reg_000_157, reg_000_158, reg_000_159 : std_logic_vector(7 downto 0);
signal reg_000_160, reg_000_161, reg_000_162, reg_000_163, reg_000_164, reg_000_165, reg_000_166, reg_000_167, reg_000_168, reg_000_169 : std_logic_vector(7 downto 0);
signal reg_000_170, reg_000_171, reg_000_172, reg_000_173, reg_000_174, reg_000_175, reg_000_176, reg_000_177, reg_000_178, reg_000_179 : std_logic_vector(7 downto 0);
signal reg_000_180, reg_000_181, reg_000_182, reg_000_183, reg_000_184, reg_000_185, reg_000_186, reg_000_187, reg_000_188, reg_000_189 : std_logic_vector(7 downto 0);
signal reg_000_190, reg_000_191, reg_000_192, reg_000_193, reg_000_194, reg_000_195, reg_000_196, reg_000_197, reg_000_198, reg_000_199 : std_logic_vector(7 downto 0);
signal reg_000_200, reg_000_201, reg_000_202, reg_000_203, reg_000_204, reg_000_205, reg_000_206, reg_000_207, reg_000_208, reg_000_209 : std_logic_vector(7 downto 0);
signal reg_000_210, reg_000_211, reg_000_212, reg_000_213, reg_000_214, reg_000_215, reg_000_216, reg_000_217, reg_000_218, reg_000_219 : std_logic_vector(7 downto 0);
signal reg_000_220, reg_000_221, reg_000_222, reg_000_223, reg_000_224, reg_000_225, reg_000_226, reg_000_227, reg_000_228, reg_000_229 : std_logic_vector(7 downto 0);
signal reg_000_230, reg_000_231, reg_000_232, reg_000_233, reg_000_234, reg_000_235, reg_000_236, reg_000_237, reg_000_238, reg_000_239 : std_logic_vector(7 downto 0);
signal reg_000_240, reg_000_241, reg_000_242, reg_000_243, reg_000_244, reg_000_245, reg_000_246, reg_000_247, reg_000_248, reg_000_249 : std_logic_vector(7 downto 0);
signal reg_000_250, reg_000_251, reg_000_252, reg_000_253, reg_000_254, reg_000_255 : std_logic_vector(7 downto 0);

signal enb_000, enb_001, enb_002, enb_003, enb_004, enb_005, enb_006, enb_007, enb_008, enb_009 : std_logic;
signal enb_010, enb_011, enb_012, enb_013, enb_014, enb_015, enb_016, enb_017, enb_018, enb_019 : std_logic;
signal enb_020, enb_021, enb_022, enb_023, enb_024, enb_025, enb_026, enb_027, enb_028, enb_029 : std_logic;
signal enb_030, enb_031, enb_032, enb_033, enb_034, enb_035, enb_036, enb_037, enb_038, enb_039 : std_logic;
signal enb_040, enb_041, enb_042, enb_043, enb_044, enb_045, enb_046, enb_047, enb_048, enb_049 : std_logic;
signal enb_050, enb_051, enb_052, enb_053, enb_054, enb_055, enb_056, enb_057, enb_058, enb_059 : std_logic;
signal enb_060, enb_061, enb_062, enb_063, enb_064, enb_065, enb_066, enb_067, enb_068, enb_069 : std_logic;
signal enb_070, enb_071, enb_072, enb_073, enb_074, enb_075, enb_076, enb_077, enb_078, enb_079 : std_logic;
signal enb_080, enb_081, enb_082, enb_083, enb_084, enb_085, enb_086, enb_087, enb_088, enb_089 : std_logic;
signal enb_090, enb_091, enb_092, enb_093, enb_094, enb_095, enb_096, enb_097, enb_098, enb_099 : std_logic;
signal enb_100, enb_101, enb_102, enb_103, enb_104, enb_105, enb_106, enb_107, enb_108, enb_109 : std_logic;
signal enb_110, enb_111, enb_112, enb_113, enb_114, enb_115, enb_116, enb_117, enb_118, enb_119 : std_logic;
signal enb_120, enb_121, enb_122, enb_123, enb_124, enb_125, enb_126, enb_127, enb_128, enb_129 : std_logic;
signal enb_130, enb_131, enb_132, enb_133, enb_134, enb_135, enb_136, enb_137, enb_138, enb_139 : std_logic;
signal enb_140, enb_141, enb_142, enb_143, enb_144, enb_145, enb_146, enb_147, enb_148, enb_149 : std_logic;
signal enb_150, enb_151, enb_152, enb_153, enb_154, enb_155, enb_156, enb_157, enb_158, enb_159 : std_logic;
signal enb_160, enb_161, enb_162, enb_163, enb_164, enb_165, enb_166, enb_167, enb_168, enb_169 : std_logic;
signal enb_170, enb_171, enb_172, enb_173, enb_174, enb_175, enb_176, enb_177, enb_178, enb_179 : std_logic;
signal enb_180, enb_181, enb_182, enb_183, enb_184, enb_185, enb_186, enb_187, enb_188, enb_189 : std_logic;
signal enb_190, enb_191, enb_192, enb_193, enb_194, enb_195, enb_196, enb_197, enb_198, enb_199 : std_logic;
signal enb_200, enb_201, enb_202, enb_203, enb_204, enb_205, enb_206, enb_207, enb_208, enb_209 : std_logic;
signal enb_210, enb_211, enb_212, enb_213, enb_214, enb_215, enb_216, enb_217, enb_218, enb_219 : std_logic;
signal enb_220, enb_221, enb_222, enb_223, enb_224, enb_225, enb_226, enb_227, enb_228, enb_229 : std_logic;
signal enb_230, enb_231, enb_232, enb_233, enb_234, enb_235, enb_236, enb_237, enb_238, enb_239 : std_logic;
signal enb_240, enb_241, enb_242, enb_243, enb_244, enb_245, enb_246, enb_247, enb_248, enb_249 : std_logic;
signal enb_250, enb_251, enb_252, enb_253, enb_254, enb_255 : std_logic;
signal CO9is1, CO128is0 : std_logic;
signal EW8Data : std_logic_vector(511 downto 0);
signal coeff_000: std_logic_vector(1 downto 0);

begin

CO9is1 <= '1' when CO9 = 1 else '0'; 
--CO128is0 <= '1' when CO128 = 1 else '0'; -- for lay15 one more dly
CO128is0 <= '1' when CO128 = 0 else '0'; -- for ex2 no more dly

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		OutChSel_dly2 <= (others => '0');
		OutChSel_dly1 <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		OutChSel_dly2 <= OutChSel_dly1;
		OutChSel_dly1 <= OutChSel;
	end if;  
end process;

EW8Sel <= OutChSel_dly2;

---------------------------------------------------------
  PixRomData <= 
X_RD_DATA( 1023 downto 1016 ) when CO128_dly2 = "0000000000" else 
X_RD_DATA( 1015 downto 1008 ) when CO128_dly2 = "0000000001" else 
X_RD_DATA( 1007 downto 1000 ) when CO128_dly2 = "0000000010" else 
X_RD_DATA( 999 downto 992 ) when CO128_dly2 = "0000000011" else 
X_RD_DATA( 991 downto 984 ) when CO128_dly2 = "0000000100" else 
X_RD_DATA( 983 downto 976 ) when CO128_dly2 = "0000000101" else 
X_RD_DATA( 975 downto 968 ) when CO128_dly2 = "0000000110" else 
X_RD_DATA( 967 downto 960 ) when CO128_dly2 = "0000000111" else 
X_RD_DATA( 959 downto 952 ) when CO128_dly2 = "0000001000" else 
X_RD_DATA( 951 downto 944 ) when CO128_dly2 = "0000001001" else 
X_RD_DATA( 943 downto 936 ) when CO128_dly2 = "0000001010" else 
X_RD_DATA( 935 downto 928 ) when CO128_dly2 = "0000001011" else 
X_RD_DATA( 927 downto 920 ) when CO128_dly2 = "0000001100" else 
X_RD_DATA( 919 downto 912 ) when CO128_dly2 = "0000001101" else 
X_RD_DATA( 911 downto 904 ) when CO128_dly2 = "0000001110" else 
X_RD_DATA( 903 downto 896 ) when CO128_dly2 = "0000001111" else 
X_RD_DATA( 895 downto 888 ) when CO128_dly2 = "0000010000" else 
X_RD_DATA( 887 downto 880 ) when CO128_dly2 = "0000010001" else 
X_RD_DATA( 879 downto 872 ) when CO128_dly2 = "0000010010" else 
X_RD_DATA( 871 downto 864 ) when CO128_dly2 = "0000010011" else 
X_RD_DATA( 863 downto 856 ) when CO128_dly2 = "0000010100" else 
X_RD_DATA( 855 downto 848 ) when CO128_dly2 = "0000010101" else 
X_RD_DATA( 847 downto 840 ) when CO128_dly2 = "0000010110" else 
X_RD_DATA( 839 downto 832 ) when CO128_dly2 = "0000010111" else 
X_RD_DATA( 831 downto 824 ) when CO128_dly2 = "0000011000" else 
X_RD_DATA( 823 downto 816 ) when CO128_dly2 = "0000011001" else 
X_RD_DATA( 815 downto 808 ) when CO128_dly2 = "0000011010" else 
X_RD_DATA( 807 downto 800 ) when CO128_dly2 = "0000011011" else 
X_RD_DATA( 799 downto 792 ) when CO128_dly2 = "0000011100" else 
X_RD_DATA( 791 downto 784 ) when CO128_dly2 = "0000011101" else 
X_RD_DATA( 783 downto 776 ) when CO128_dly2 = "0000011110" else 
X_RD_DATA( 775 downto 768 ) when CO128_dly2 = "0000011111" else 
X_RD_DATA( 767 downto 760 ) when CO128_dly2 = "0000100000" else 
X_RD_DATA( 759 downto 752 ) when CO128_dly2 = "0000100001" else 
X_RD_DATA( 751 downto 744 ) when CO128_dly2 = "0000100010" else 
X_RD_DATA( 743 downto 736 ) when CO128_dly2 = "0000100011" else 
X_RD_DATA( 735 downto 728 ) when CO128_dly2 = "0000100100" else 
X_RD_DATA( 727 downto 720 ) when CO128_dly2 = "0000100101" else 
X_RD_DATA( 719 downto 712 ) when CO128_dly2 = "0000100110" else 
X_RD_DATA( 711 downto 704 ) when CO128_dly2 = "0000100111" else 
X_RD_DATA( 703 downto 696 ) when CO128_dly2 = "0000101000" else 
X_RD_DATA( 695 downto 688 ) when CO128_dly2 = "0000101001" else 
X_RD_DATA( 687 downto 680 ) when CO128_dly2 = "0000101010" else 
X_RD_DATA( 679 downto 672 ) when CO128_dly2 = "0000101011" else 
X_RD_DATA( 671 downto 664 ) when CO128_dly2 = "0000101100" else 
X_RD_DATA( 663 downto 656 ) when CO128_dly2 = "0000101101" else 
X_RD_DATA( 655 downto 648 ) when CO128_dly2 = "0000101110" else 
X_RD_DATA( 647 downto 640 ) when CO128_dly2 = "0000101111" else 
X_RD_DATA( 639 downto 632 ) when CO128_dly2 = "0000110000" else 
X_RD_DATA( 631 downto 624 ) when CO128_dly2 = "0000110001" else 
X_RD_DATA( 623 downto 616 ) when CO128_dly2 = "0000110010" else 
X_RD_DATA( 615 downto 608 ) when CO128_dly2 = "0000110011" else 
X_RD_DATA( 607 downto 600 ) when CO128_dly2 = "0000110100" else 
X_RD_DATA( 599 downto 592 ) when CO128_dly2 = "0000110101" else 
X_RD_DATA( 591 downto 584 ) when CO128_dly2 = "0000110110" else 
X_RD_DATA( 583 downto 576 ) when CO128_dly2 = "0000110111" else 
X_RD_DATA( 575 downto 568 ) when CO128_dly2 = "0000111000" else 
X_RD_DATA( 567 downto 560 ) when CO128_dly2 = "0000111001" else 
X_RD_DATA( 559 downto 552 ) when CO128_dly2 = "0000111010" else 
X_RD_DATA( 551 downto 544 ) when CO128_dly2 = "0000111011" else 
X_RD_DATA( 543 downto 536 ) when CO128_dly2 = "0000111100" else 
X_RD_DATA( 535 downto 528 ) when CO128_dly2 = "0000111101" else 
X_RD_DATA( 527 downto 520 ) when CO128_dly2 = "0000111110" else 
X_RD_DATA( 519 downto 512 ) when CO128_dly2 = "0000111111" else 
X_RD_DATA( 511 downto 504 ) when CO128_dly2 = "0001000000" else 
X_RD_DATA( 503 downto 496 ) when CO128_dly2 = "0001000001" else 
X_RD_DATA( 495 downto 488 ) when CO128_dly2 = "0001000010" else 
X_RD_DATA( 487 downto 480 ) when CO128_dly2 = "0001000011" else 
X_RD_DATA( 479 downto 472 ) when CO128_dly2 = "0001000100" else 
X_RD_DATA( 471 downto 464 ) when CO128_dly2 = "0001000101" else 
X_RD_DATA( 463 downto 456 ) when CO128_dly2 = "0001000110" else 
X_RD_DATA( 455 downto 448 ) when CO128_dly2 = "0001000111" else 
X_RD_DATA( 447 downto 440 ) when CO128_dly2 = "0001001000" else 
X_RD_DATA( 439 downto 432 ) when CO128_dly2 = "0001001001" else 
X_RD_DATA( 431 downto 424 ) when CO128_dly2 = "0001001010" else 
X_RD_DATA( 423 downto 416 ) when CO128_dly2 = "0001001011" else 
X_RD_DATA( 415 downto 408 ) when CO128_dly2 = "0001001100" else 
X_RD_DATA( 407 downto 400 ) when CO128_dly2 = "0001001101" else 
X_RD_DATA( 399 downto 392 ) when CO128_dly2 = "0001001110" else 
X_RD_DATA( 391 downto 384 ) when CO128_dly2 = "0001001111" else 
X_RD_DATA( 383 downto 376 ) when CO128_dly2 = "0001010000" else 
X_RD_DATA( 375 downto 368 ) when CO128_dly2 = "0001010001" else 
X_RD_DATA( 367 downto 360 ) when CO128_dly2 = "0001010010" else 
X_RD_DATA( 359 downto 352 ) when CO128_dly2 = "0001010011" else 
X_RD_DATA( 351 downto 344 ) when CO128_dly2 = "0001010100" else 
X_RD_DATA( 343 downto 336 ) when CO128_dly2 = "0001010101" else 
X_RD_DATA( 335 downto 328 ) when CO128_dly2 = "0001010110" else 
X_RD_DATA( 327 downto 320 ) when CO128_dly2 = "0001010111" else 
X_RD_DATA( 319 downto 312 ) when CO128_dly2 = "0001011000" else 
X_RD_DATA( 311 downto 304 ) when CO128_dly2 = "0001011001" else 
X_RD_DATA( 303 downto 296 ) when CO128_dly2 = "0001011010" else 
X_RD_DATA( 295 downto 288 ) when CO128_dly2 = "0001011011" else 
X_RD_DATA( 287 downto 280 ) when CO128_dly2 = "0001011100" else 
X_RD_DATA( 279 downto 272 ) when CO128_dly2 = "0001011101" else 
X_RD_DATA( 271 downto 264 ) when CO128_dly2 = "0001011110" else 
X_RD_DATA( 263 downto 256 ) when CO128_dly2 = "0001011111" else 
X_RD_DATA( 255 downto 248 ) when CO128_dly2 = "0001100000" else 
X_RD_DATA( 247 downto 240 ) when CO128_dly2 = "0001100001" else 
X_RD_DATA( 239 downto 232 ) when CO128_dly2 = "0001100010" else 
X_RD_DATA( 231 downto 224 ) when CO128_dly2 = "0001100011" else 
X_RD_DATA( 223 downto 216 ) when CO128_dly2 = "0001100100" else 
X_RD_DATA( 215 downto 208 ) when CO128_dly2 = "0001100101" else 
X_RD_DATA( 207 downto 200 ) when CO128_dly2 = "0001100110" else 
X_RD_DATA( 199 downto 192 ) when CO128_dly2 = "0001100111" else 
X_RD_DATA( 191 downto 184 ) when CO128_dly2 = "0001101000" else 
X_RD_DATA( 183 downto 176 ) when CO128_dly2 = "0001101001" else 
X_RD_DATA( 175 downto 168 ) when CO128_dly2 = "0001101010" else 
X_RD_DATA( 167 downto 160 ) when CO128_dly2 = "0001101011" else 
X_RD_DATA( 159 downto 152 ) when CO128_dly2 = "0001101100" else 
X_RD_DATA( 151 downto 144 ) when CO128_dly2 = "0001101101" else 
X_RD_DATA( 143 downto 136 ) when CO128_dly2 = "0001101110" else 
X_RD_DATA( 135 downto 128 ) when CO128_dly2 = "0001101111" else 
X_RD_DATA( 127 downto 120 ) when CO128_dly2 = "0001110000" else 
X_RD_DATA( 119 downto 112 ) when CO128_dly2 = "0001110001" else 
X_RD_DATA( 111 downto 104 ) when CO128_dly2 = "0001110010" else 
X_RD_DATA( 103 downto 96 ) when CO128_dly2 = "0001110011" else 
X_RD_DATA( 95 downto 88 ) when CO128_dly2 = "0001110100" else 
X_RD_DATA( 87 downto 80 ) when CO128_dly2 = "0001110101" else 
X_RD_DATA( 79 downto 72 ) when CO128_dly2 = "0001110110" else 
X_RD_DATA( 71 downto 64 ) when CO128_dly2 = "0001110111" else 
X_RD_DATA( 63 downto 56 ) when CO128_dly2 = "0001111000" else 
X_RD_DATA( 55 downto 48 ) when CO128_dly2 = "0001111001" else 
X_RD_DATA( 47 downto 40 ) when CO128_dly2 = "0001111010" else 
X_RD_DATA( 39 downto 32 ) when CO128_dly2 = "0001111011" else 
X_RD_DATA( 31 downto 24 ) when CO128_dly2 = "0001111100" else 
X_RD_DATA( 23 downto 16 ) when CO128_dly2 = "0001111101" else 
X_RD_DATA( 15 downto 8 ) when CO128_dly2 = "0001111110" else 
X_RD_DATA( 7 downto 0 ); -- when CO128_dly2 = "0001111111" else 


EW8Addr_L <= CO9;
EW8Addr_H <= (CO128(7 downto 0) & "000") + ("0" & CO128); -- CO128 x 9
EW8Addr_prev <= EW8Addr_H + ("0000000" & EW8Addr_L);

--EW8Addr_prev <= (CO128); -- CO128 x 1

---------------------------------------------------------
process( RST_N, CLK )
begin
        if( RST_N = '0' ) then
                EW8Addr <= (others => '0');
        elsif( CLK'event and CLK = '1' ) then
                EW8Addr <= EW8Addr_prev;
        end if; 
end process;


---------------------------------------------------------
enb_000 <= '1' when EW8Sel = 1 and CO128 = 127 else '0';
enb_001 <= '1' when EW8Sel = 2 and CO128 = 127 else '0';
enb_002 <= '1' when EW8Sel = 3 and CO128 = 127 else '0';
enb_003 <= '1' when EW8Sel = 4 and CO128 = 127 else '0';
enb_004 <= '1' when EW8Sel = 5 and CO128 = 127 else '0';
enb_005 <= '1' when EW8Sel = 6 and CO128 = 127 else '0';
enb_006 <= '1' when EW8Sel = 7 and CO128 = 127 else '0';
enb_007 <= '1' when EW8Sel = 8 and CO128 = 127 else '0';
enb_008 <= '1' when EW8Sel = 9 and CO128 = 127 else '0';
enb_009 <= '1' when EW8Sel = 10 and CO128 = 127 else '0';
enb_010 <= '1' when EW8Sel = 11 and CO128 = 127 else '0';
enb_011 <= '1' when EW8Sel = 12 and CO128 = 127 else '0';
enb_012 <= '1' when EW8Sel = 13 and CO128 = 127 else '0';
enb_013 <= '1' when EW8Sel = 14 and CO128 = 127 else '0';
enb_014 <= '1' when EW8Sel = 15 and CO128 = 127 else '0';
enb_015 <= '1' when EW8Sel = 16 and CO128 = 127 else '0';
enb_016 <= '1' when EW8Sel = 17 and CO128 = 127 else '0';
enb_017 <= '1' when EW8Sel = 18 and CO128 = 127 else '0';
enb_018 <= '1' when EW8Sel = 19 and CO128 = 127 else '0';
enb_019 <= '1' when EW8Sel = 20 and CO128 = 127 else '0';
enb_020 <= '1' when EW8Sel = 21 and CO128 = 127 else '0';
enb_021 <= '1' when EW8Sel = 22 and CO128 = 127 else '0';
enb_022 <= '1' when EW8Sel = 23 and CO128 = 127 else '0';
enb_023 <= '1' when EW8Sel = 24 and CO128 = 127 else '0';
enb_024 <= '1' when EW8Sel = 25 and CO128 = 127 else '0';
enb_025 <= '1' when EW8Sel = 26 and CO128 = 127 else '0';
enb_026 <= '1' when EW8Sel = 27 and CO128 = 127 else '0';
enb_027 <= '1' when EW8Sel = 28 and CO128 = 127 else '0';
enb_028 <= '1' when EW8Sel = 29 and CO128 = 127 else '0';
enb_029 <= '1' when EW8Sel = 30 and CO128 = 127 else '0';
enb_030 <= '1' when EW8Sel = 31 and CO128 = 127 else '0';
enb_031 <= '1' when EW8Sel = 32 and CO128 = 127 else '0';
enb_032 <= '1' when EW8Sel = 33 and CO128 = 127 else '0';
enb_033 <= '1' when EW8Sel = 34 and CO128 = 127 else '0';
enb_034 <= '1' when EW8Sel = 35 and CO128 = 127 else '0';
enb_035 <= '1' when EW8Sel = 36 and CO128 = 127 else '0';
enb_036 <= '1' when EW8Sel = 37 and CO128 = 127 else '0';
enb_037 <= '1' when EW8Sel = 38 and CO128 = 127 else '0';
enb_038 <= '1' when EW8Sel = 39 and CO128 = 127 else '0';
enb_039 <= '1' when EW8Sel = 40 and CO128 = 127 else '0';
enb_040 <= '1' when EW8Sel = 41 and CO128 = 127 else '0';
enb_041 <= '1' when EW8Sel = 42 and CO128 = 127 else '0';
enb_042 <= '1' when EW8Sel = 43 and CO128 = 127 else '0';
enb_043 <= '1' when EW8Sel = 44 and CO128 = 127 else '0';
enb_044 <= '1' when EW8Sel = 45 and CO128 = 127 else '0';
enb_045 <= '1' when EW8Sel = 46 and CO128 = 127 else '0';
enb_046 <= '1' when EW8Sel = 47 and CO128 = 127 else '0';
enb_047 <= '1' when EW8Sel = 48 and CO128 = 127 else '0';
enb_048 <= '1' when EW8Sel = 49 and CO128 = 127 else '0';
enb_049 <= '1' when EW8Sel = 50 and CO128 = 127 else '0';
enb_050 <= '1' when EW8Sel = 51 and CO128 = 127 else '0';
enb_051 <= '1' when EW8Sel = 52 and CO128 = 127 else '0';
enb_052 <= '1' when EW8Sel = 53 and CO128 = 127 else '0';
enb_053 <= '1' when EW8Sel = 54 and CO128 = 127 else '0';
enb_054 <= '1' when EW8Sel = 55 and CO128 = 127 else '0';
enb_055 <= '1' when EW8Sel = 56 and CO128 = 127 else '0';
enb_056 <= '1' when EW8Sel = 57 and CO128 = 127 else '0';
enb_057 <= '1' when EW8Sel = 58 and CO128 = 127 else '0';
enb_058 <= '1' when EW8Sel = 59 and CO128 = 127 else '0';
enb_059 <= '1' when EW8Sel = 60 and CO128 = 127 else '0';
enb_060 <= '1' when EW8Sel = 61 and CO128 = 127 else '0';
enb_061 <= '1' when EW8Sel = 62 and CO128 = 127 else '0';
enb_062 <= '1' when EW8Sel = 63 and CO128 = 127 else '0';
enb_063 <= '1' when EW8Sel = 64 and CO128 = 127 else '0';
enb_064 <= '1' when EW8Sel = 65 and CO128 = 127 else '0';
enb_065 <= '1' when EW8Sel = 66 and CO128 = 127 else '0';
enb_066 <= '1' when EW8Sel = 67 and CO128 = 127 else '0';
enb_067 <= '1' when EW8Sel = 68 and CO128 = 127 else '0';
enb_068 <= '1' when EW8Sel = 69 and CO128 = 127 else '0';
enb_069 <= '1' when EW8Sel = 70 and CO128 = 127 else '0';
enb_070 <= '1' when EW8Sel = 71 and CO128 = 127 else '0';
enb_071 <= '1' when EW8Sel = 72 and CO128 = 127 else '0';
enb_072 <= '1' when EW8Sel = 73 and CO128 = 127 else '0';
enb_073 <= '1' when EW8Sel = 74 and CO128 = 127 else '0';
enb_074 <= '1' when EW8Sel = 75 and CO128 = 127 else '0';
enb_075 <= '1' when EW8Sel = 76 and CO128 = 127 else '0';
enb_076 <= '1' when EW8Sel = 77 and CO128 = 127 else '0';
enb_077 <= '1' when EW8Sel = 78 and CO128 = 127 else '0';
enb_078 <= '1' when EW8Sel = 79 and CO128 = 127 else '0';
enb_079 <= '1' when EW8Sel = 80 and CO128 = 127 else '0';
enb_080 <= '1' when EW8Sel = 81 and CO128 = 127 else '0';
enb_081 <= '1' when EW8Sel = 82 and CO128 = 127 else '0';
enb_082 <= '1' when EW8Sel = 83 and CO128 = 127 else '0';
enb_083 <= '1' when EW8Sel = 84 and CO128 = 127 else '0';
enb_084 <= '1' when EW8Sel = 85 and CO128 = 127 else '0';
enb_085 <= '1' when EW8Sel = 86 and CO128 = 127 else '0';
enb_086 <= '1' when EW8Sel = 87 and CO128 = 127 else '0';
enb_087 <= '1' when EW8Sel = 88 and CO128 = 127 else '0';
enb_088 <= '1' when EW8Sel = 89 and CO128 = 127 else '0';
enb_089 <= '1' when EW8Sel = 90 and CO128 = 127 else '0';
enb_090 <= '1' when EW8Sel = 91 and CO128 = 127 else '0';
enb_091 <= '1' when EW8Sel = 92 and CO128 = 127 else '0';
enb_092 <= '1' when EW8Sel = 93 and CO128 = 127 else '0';
enb_093 <= '1' when EW8Sel = 94 and CO128 = 127 else '0';
enb_094 <= '1' when EW8Sel = 95 and CO128 = 127 else '0';
enb_095 <= '1' when EW8Sel = 96 and CO128 = 127 else '0';
enb_096 <= '1' when EW8Sel = 97 and CO128 = 127 else '0';
enb_097 <= '1' when EW8Sel = 98 and CO128 = 127 else '0';
enb_098 <= '1' when EW8Sel = 99 and CO128 = 127 else '0';
enb_099 <= '1' when EW8Sel = 100 and CO128 = 127 else '0';
enb_100 <= '1' when EW8Sel = 101 and CO128 = 127 else '0';
enb_101 <= '1' when EW8Sel = 102 and CO128 = 127 else '0';
enb_102 <= '1' when EW8Sel = 103 and CO128 = 127 else '0';
enb_103 <= '1' when EW8Sel = 104 and CO128 = 127 else '0';
enb_104 <= '1' when EW8Sel = 105 and CO128 = 127 else '0';
enb_105 <= '1' when EW8Sel = 106 and CO128 = 127 else '0';
enb_106 <= '1' when EW8Sel = 107 and CO128 = 127 else '0';
enb_107 <= '1' when EW8Sel = 108 and CO128 = 127 else '0';
enb_108 <= '1' when EW8Sel = 109 and CO128 = 127 else '0';
enb_109 <= '1' when EW8Sel = 110 and CO128 = 127 else '0';
enb_110 <= '1' when EW8Sel = 111 and CO128 = 127 else '0';
enb_111 <= '1' when EW8Sel = 112 and CO128 = 127 else '0';
enb_112 <= '1' when EW8Sel = 113 and CO128 = 127 else '0';
enb_113 <= '1' when EW8Sel = 114 and CO128 = 127 else '0';
enb_114 <= '1' when EW8Sel = 115 and CO128 = 127 else '0';
enb_115 <= '1' when EW8Sel = 116 and CO128 = 127 else '0';
enb_116 <= '1' when EW8Sel = 117 and CO128 = 127 else '0';
enb_117 <= '1' when EW8Sel = 118 and CO128 = 127 else '0';
enb_118 <= '1' when EW8Sel = 119 and CO128 = 127 else '0';
enb_119 <= '1' when EW8Sel = 120 and CO128 = 127 else '0';
enb_120 <= '1' when EW8Sel = 121 and CO128 = 127 else '0';
enb_121 <= '1' when EW8Sel = 122 and CO128 = 127 else '0';
enb_122 <= '1' when EW8Sel = 123 and CO128 = 127 else '0';
enb_123 <= '1' when EW8Sel = 124 and CO128 = 127 else '0';
enb_124 <= '1' when EW8Sel = 125 and CO128 = 127 else '0';
enb_125 <= '1' when EW8Sel = 126 and CO128 = 127 else '0';
enb_126 <= '1' when EW8Sel = 127 and CO128 = 127 else '0';
enb_127 <= '1' when EW8Sel = 128 and CO128 = 127 else '0';
enb_128 <= '1' when EW8Sel = 129 and CO128 = 127 else '0';
enb_129 <= '1' when EW8Sel = 130 and CO128 = 127 else '0';
enb_130 <= '1' when EW8Sel = 131 and CO128 = 127 else '0';
enb_131 <= '1' when EW8Sel = 132 and CO128 = 127 else '0';
enb_132 <= '1' when EW8Sel = 133 and CO128 = 127 else '0';
enb_133 <= '1' when EW8Sel = 134 and CO128 = 127 else '0';
enb_134 <= '1' when EW8Sel = 135 and CO128 = 127 else '0';
enb_135 <= '1' when EW8Sel = 136 and CO128 = 127 else '0';
enb_136 <= '1' when EW8Sel = 137 and CO128 = 127 else '0';
enb_137 <= '1' when EW8Sel = 138 and CO128 = 127 else '0';
enb_138 <= '1' when EW8Sel = 139 and CO128 = 127 else '0';
enb_139 <= '1' when EW8Sel = 140 and CO128 = 127 else '0';
enb_140 <= '1' when EW8Sel = 141 and CO128 = 127 else '0';
enb_141 <= '1' when EW8Sel = 142 and CO128 = 127 else '0';
enb_142 <= '1' when EW8Sel = 143 and CO128 = 127 else '0';
enb_143 <= '1' when EW8Sel = 144 and CO128 = 127 else '0';
enb_144 <= '1' when EW8Sel = 145 and CO128 = 127 else '0';
enb_145 <= '1' when EW8Sel = 146 and CO128 = 127 else '0';
enb_146 <= '1' when EW8Sel = 147 and CO128 = 127 else '0';
enb_147 <= '1' when EW8Sel = 148 and CO128 = 127 else '0';
enb_148 <= '1' when EW8Sel = 149 and CO128 = 127 else '0';
enb_149 <= '1' when EW8Sel = 150 and CO128 = 127 else '0';
enb_150 <= '1' when EW8Sel = 151 and CO128 = 127 else '0';
enb_151 <= '1' when EW8Sel = 152 and CO128 = 127 else '0';
enb_152 <= '1' when EW8Sel = 153 and CO128 = 127 else '0';
enb_153 <= '1' when EW8Sel = 154 and CO128 = 127 else '0';
enb_154 <= '1' when EW8Sel = 155 and CO128 = 127 else '0';
enb_155 <= '1' when EW8Sel = 156 and CO128 = 127 else '0';
enb_156 <= '1' when EW8Sel = 157 and CO128 = 127 else '0';
enb_157 <= '1' when EW8Sel = 158 and CO128 = 127 else '0';
enb_158 <= '1' when EW8Sel = 159 and CO128 = 127 else '0';
enb_159 <= '1' when EW8Sel = 160 and CO128 = 127 else '0';
enb_160 <= '1' when EW8Sel = 161 and CO128 = 127 else '0';
enb_161 <= '1' when EW8Sel = 162 and CO128 = 127 else '0';
enb_162 <= '1' when EW8Sel = 163 and CO128 = 127 else '0';
enb_163 <= '1' when EW8Sel = 164 and CO128 = 127 else '0';
enb_164 <= '1' when EW8Sel = 165 and CO128 = 127 else '0';
enb_165 <= '1' when EW8Sel = 166 and CO128 = 127 else '0';
enb_166 <= '1' when EW8Sel = 167 and CO128 = 127 else '0';
enb_167 <= '1' when EW8Sel = 168 and CO128 = 127 else '0';
enb_168 <= '1' when EW8Sel = 169 and CO128 = 127 else '0';
enb_169 <= '1' when EW8Sel = 170 and CO128 = 127 else '0';
enb_170 <= '1' when EW8Sel = 171 and CO128 = 127 else '0';
enb_171 <= '1' when EW8Sel = 172 and CO128 = 127 else '0';
enb_172 <= '1' when EW8Sel = 173 and CO128 = 127 else '0';
enb_173 <= '1' when EW8Sel = 174 and CO128 = 127 else '0';
enb_174 <= '1' when EW8Sel = 175 and CO128 = 127 else '0';
enb_175 <= '1' when EW8Sel = 176 and CO128 = 127 else '0';
enb_176 <= '1' when EW8Sel = 177 and CO128 = 127 else '0';
enb_177 <= '1' when EW8Sel = 178 and CO128 = 127 else '0';
enb_178 <= '1' when EW8Sel = 179 and CO128 = 127 else '0';
enb_179 <= '1' when EW8Sel = 180 and CO128 = 127 else '0';
enb_180 <= '1' when EW8Sel = 181 and CO128 = 127 else '0';
enb_181 <= '1' when EW8Sel = 182 and CO128 = 127 else '0';
enb_182 <= '1' when EW8Sel = 183 and CO128 = 127 else '0';
enb_183 <= '1' when EW8Sel = 184 and CO128 = 127 else '0';
enb_184 <= '1' when EW8Sel = 185 and CO128 = 127 else '0';
enb_185 <= '1' when EW8Sel = 186 and CO128 = 127 else '0';
enb_186 <= '1' when EW8Sel = 187 and CO128 = 127 else '0';
enb_187 <= '1' when EW8Sel = 188 and CO128 = 127 else '0';
enb_188 <= '1' when EW8Sel = 189 and CO128 = 127 else '0';
enb_189 <= '1' when EW8Sel = 190 and CO128 = 127 else '0';
enb_190 <= '1' when EW8Sel = 191 and CO128 = 127 else '0';
enb_191 <= '1' when EW8Sel = 192 and CO128 = 127 else '0';
enb_192 <= '1' when EW8Sel = 193 and CO128 = 127 else '0';
enb_193 <= '1' when EW8Sel = 194 and CO128 = 127 else '0';
enb_194 <= '1' when EW8Sel = 195 and CO128 = 127 else '0';
enb_195 <= '1' when EW8Sel = 196 and CO128 = 127 else '0';
enb_196 <= '1' when EW8Sel = 197 and CO128 = 127 else '0';
enb_197 <= '1' when EW8Sel = 198 and CO128 = 127 else '0';
enb_198 <= '1' when EW8Sel = 199 and CO128 = 127 else '0';
enb_199 <= '1' when EW8Sel = 200 and CO128 = 127 else '0';
enb_200 <= '1' when EW8Sel = 201 and CO128 = 127 else '0';
enb_201 <= '1' when EW8Sel = 202 and CO128 = 127 else '0';
enb_202 <= '1' when EW8Sel = 203 and CO128 = 127 else '0';
enb_203 <= '1' when EW8Sel = 204 and CO128 = 127 else '0';
enb_204 <= '1' when EW8Sel = 205 and CO128 = 127 else '0';
enb_205 <= '1' when EW8Sel = 206 and CO128 = 127 else '0';
enb_206 <= '1' when EW8Sel = 207 and CO128 = 127 else '0';
enb_207 <= '1' when EW8Sel = 208 and CO128 = 127 else '0';
enb_208 <= '1' when EW8Sel = 209 and CO128 = 127 else '0';
enb_209 <= '1' when EW8Sel = 210 and CO128 = 127 else '0';
enb_210 <= '1' when EW8Sel = 211 and CO128 = 127 else '0';
enb_211 <= '1' when EW8Sel = 212 and CO128 = 127 else '0';
enb_212 <= '1' when EW8Sel = 213 and CO128 = 127 else '0';
enb_213 <= '1' when EW8Sel = 214 and CO128 = 127 else '0';
enb_214 <= '1' when EW8Sel = 215 and CO128 = 127 else '0';
enb_215 <= '1' when EW8Sel = 216 and CO128 = 127 else '0';
enb_216 <= '1' when EW8Sel = 217 and CO128 = 127 else '0';
enb_217 <= '1' when EW8Sel = 218 and CO128 = 127 else '0';
enb_218 <= '1' when EW8Sel = 219 and CO128 = 127 else '0';
enb_219 <= '1' when EW8Sel = 220 and CO128 = 127 else '0';
enb_220 <= '1' when EW8Sel = 221 and CO128 = 127 else '0';
enb_221 <= '1' when EW8Sel = 222 and CO128 = 127 else '0';
enb_222 <= '1' when EW8Sel = 223 and CO128 = 127 else '0';
enb_223 <= '1' when EW8Sel = 224 and CO128 = 127 else '0';
enb_224 <= '1' when EW8Sel = 225 and CO128 = 127 else '0';
enb_225 <= '1' when EW8Sel = 226 and CO128 = 127 else '0';
enb_226 <= '1' when EW8Sel = 227 and CO128 = 127 else '0';
enb_227 <= '1' when EW8Sel = 228 and CO128 = 127 else '0';
enb_228 <= '1' when EW8Sel = 229 and CO128 = 127 else '0';
enb_229 <= '1' when EW8Sel = 230 and CO128 = 127 else '0';
enb_230 <= '1' when EW8Sel = 231 and CO128 = 127 else '0';
enb_231 <= '1' when EW8Sel = 232 and CO128 = 127 else '0';
enb_232 <= '1' when EW8Sel = 233 and CO128 = 127 else '0';
enb_233 <= '1' when EW8Sel = 234 and CO128 = 127 else '0';
enb_234 <= '1' when EW8Sel = 235 and CO128 = 127 else '0';
enb_235 <= '1' when EW8Sel = 236 and CO128 = 127 else '0';
enb_236 <= '1' when EW8Sel = 237 and CO128 = 127 else '0';
enb_237 <= '1' when EW8Sel = 238 and CO128 = 127 else '0';
enb_238 <= '1' when EW8Sel = 239 and CO128 = 127 else '0';
enb_239 <= '1' when EW8Sel = 240 and CO128 = 127 else '0';
enb_240 <= '1' when EW8Sel = 241 and CO128 = 127 else '0';
enb_241 <= '1' when EW8Sel = 242 and CO128 = 127 else '0';
enb_242 <= '1' when EW8Sel = 243 and CO128 = 127 else '0';
enb_243 <= '1' when EW8Sel = 244 and CO128 = 127 else '0';
enb_244 <= '1' when EW8Sel = 245 and CO128 = 127 else '0';
enb_245 <= '1' when EW8Sel = 246 and CO128 = 127 else '0';
enb_246 <= '1' when EW8Sel = 247 and CO128 = 127 else '0';
enb_247 <= '1' when EW8Sel = 248 and CO128 = 127 else '0';
enb_248 <= '1' when EW8Sel = 249 and CO128 = 127 else '0';
enb_249 <= '1' when EW8Sel = 250 and CO128 = 127 else '0';
enb_250 <= '1' when EW8Sel = 251 and CO128 = 127 else '0';
enb_251 <= '1' when EW8Sel = 252 and CO128 = 127 else '0';
enb_252 <= '1' when EW8Sel = 253 and CO128 = 127 else '0';
enb_253 <= '1' when EW8Sel = 254 and CO128 = 127 else '0';
enb_254 <= '1' when EW8Sel = 255 and CO128 = 127 else '0';
enb_255 <= '1' when EW8Sel = 0 and CO128 = 127 else '0';

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
  if( CLK'event and CLK = '1' ) then
    if(enb_000 = '1') then reg_000_000 <= Relu_000;
    elsif(enb_001 = '1') then reg_000_001 <= Relu_000;
    elsif(enb_002 = '1') then reg_000_002 <= Relu_000;
    elsif(enb_003 = '1') then reg_000_003 <= Relu_000;
    elsif(enb_004 = '1') then reg_000_004 <= Relu_000;
    elsif(enb_005 = '1') then reg_000_005 <= Relu_000;
    elsif(enb_006 = '1') then reg_000_006 <= Relu_000;
    elsif(enb_007 = '1') then reg_000_007 <= Relu_000;
    elsif(enb_008 = '1') then reg_000_008 <= Relu_000;
    elsif(enb_009 = '1') then reg_000_009 <= Relu_000;
    elsif(enb_010 = '1') then reg_000_010 <= Relu_000;
    elsif(enb_011 = '1') then reg_000_011 <= Relu_000;
    elsif(enb_012 = '1') then reg_000_012 <= Relu_000;
    elsif(enb_013 = '1') then reg_000_013 <= Relu_000;
    elsif(enb_014 = '1') then reg_000_014 <= Relu_000;
    elsif(enb_015 = '1') then reg_000_015 <= Relu_000;
    elsif(enb_016 = '1') then reg_000_016 <= Relu_000;
    elsif(enb_017 = '1') then reg_000_017 <= Relu_000;
    elsif(enb_018 = '1') then reg_000_018 <= Relu_000;
    elsif(enb_019 = '1') then reg_000_019 <= Relu_000;
    elsif(enb_020 = '1') then reg_000_020 <= Relu_000;
    elsif(enb_021 = '1') then reg_000_021 <= Relu_000;
    elsif(enb_022 = '1') then reg_000_022 <= Relu_000;
    elsif(enb_023 = '1') then reg_000_023 <= Relu_000;
    elsif(enb_024 = '1') then reg_000_024 <= Relu_000;
    elsif(enb_025 = '1') then reg_000_025 <= Relu_000;
    elsif(enb_026 = '1') then reg_000_026 <= Relu_000;
    elsif(enb_027 = '1') then reg_000_027 <= Relu_000;
    elsif(enb_028 = '1') then reg_000_028 <= Relu_000;
    elsif(enb_029 = '1') then reg_000_029 <= Relu_000;
    elsif(enb_030 = '1') then reg_000_030 <= Relu_000;
    elsif(enb_031 = '1') then reg_000_031 <= Relu_000;
    elsif(enb_032 = '1') then reg_000_032 <= Relu_000;
    elsif(enb_033 = '1') then reg_000_033 <= Relu_000;
    elsif(enb_034 = '1') then reg_000_034 <= Relu_000;
    elsif(enb_035 = '1') then reg_000_035 <= Relu_000;
    elsif(enb_036 = '1') then reg_000_036 <= Relu_000;
    elsif(enb_037 = '1') then reg_000_037 <= Relu_000;
    elsif(enb_038 = '1') then reg_000_038 <= Relu_000;
    elsif(enb_039 = '1') then reg_000_039 <= Relu_000;
    elsif(enb_040 = '1') then reg_000_040 <= Relu_000;
    elsif(enb_041 = '1') then reg_000_041 <= Relu_000;
    elsif(enb_042 = '1') then reg_000_042 <= Relu_000;
    elsif(enb_043 = '1') then reg_000_043 <= Relu_000;
    elsif(enb_044 = '1') then reg_000_044 <= Relu_000;
    elsif(enb_045 = '1') then reg_000_045 <= Relu_000;
    elsif(enb_046 = '1') then reg_000_046 <= Relu_000;
    elsif(enb_047 = '1') then reg_000_047 <= Relu_000;
    elsif(enb_048 = '1') then reg_000_048 <= Relu_000;
    elsif(enb_049 = '1') then reg_000_049 <= Relu_000;
    elsif(enb_050 = '1') then reg_000_050 <= Relu_000;
    elsif(enb_051 = '1') then reg_000_051 <= Relu_000;
    elsif(enb_052 = '1') then reg_000_052 <= Relu_000;
    elsif(enb_053 = '1') then reg_000_053 <= Relu_000;
    elsif(enb_054 = '1') then reg_000_054 <= Relu_000;
    elsif(enb_055 = '1') then reg_000_055 <= Relu_000;
    elsif(enb_056 = '1') then reg_000_056 <= Relu_000;
    elsif(enb_057 = '1') then reg_000_057 <= Relu_000;
    elsif(enb_058 = '1') then reg_000_058 <= Relu_000;
    elsif(enb_059 = '1') then reg_000_059 <= Relu_000;
    elsif(enb_060 = '1') then reg_000_060 <= Relu_000;
    elsif(enb_061 = '1') then reg_000_061 <= Relu_000;
    elsif(enb_062 = '1') then reg_000_062 <= Relu_000;
    elsif(enb_063 = '1') then reg_000_063 <= Relu_000;
    elsif(enb_064 = '1') then reg_000_064 <= Relu_000;
    elsif(enb_065 = '1') then reg_000_065 <= Relu_000;
    elsif(enb_066 = '1') then reg_000_066 <= Relu_000;
    elsif(enb_067 = '1') then reg_000_067 <= Relu_000;
    elsif(enb_068 = '1') then reg_000_068 <= Relu_000;
    elsif(enb_069 = '1') then reg_000_069 <= Relu_000;
    elsif(enb_070 = '1') then reg_000_070 <= Relu_000;
    elsif(enb_071 = '1') then reg_000_071 <= Relu_000;
    elsif(enb_072 = '1') then reg_000_072 <= Relu_000;
    elsif(enb_073 = '1') then reg_000_073 <= Relu_000;
    elsif(enb_074 = '1') then reg_000_074 <= Relu_000;
    elsif(enb_075 = '1') then reg_000_075 <= Relu_000;
    elsif(enb_076 = '1') then reg_000_076 <= Relu_000;
    elsif(enb_077 = '1') then reg_000_077 <= Relu_000;
    elsif(enb_078 = '1') then reg_000_078 <= Relu_000;
    elsif(enb_079 = '1') then reg_000_079 <= Relu_000;
    elsif(enb_080 = '1') then reg_000_080 <= Relu_000;
    elsif(enb_081 = '1') then reg_000_081 <= Relu_000;
    elsif(enb_082 = '1') then reg_000_082 <= Relu_000;
    elsif(enb_083 = '1') then reg_000_083 <= Relu_000;
    elsif(enb_084 = '1') then reg_000_084 <= Relu_000;
    elsif(enb_085 = '1') then reg_000_085 <= Relu_000;
    elsif(enb_086 = '1') then reg_000_086 <= Relu_000;
    elsif(enb_087 = '1') then reg_000_087 <= Relu_000;
    elsif(enb_088 = '1') then reg_000_088 <= Relu_000;
    elsif(enb_089 = '1') then reg_000_089 <= Relu_000;
    elsif(enb_090 = '1') then reg_000_090 <= Relu_000;
    elsif(enb_091 = '1') then reg_000_091 <= Relu_000;
    elsif(enb_092 = '1') then reg_000_092 <= Relu_000;
    elsif(enb_093 = '1') then reg_000_093 <= Relu_000;
    elsif(enb_094 = '1') then reg_000_094 <= Relu_000;
    elsif(enb_095 = '1') then reg_000_095 <= Relu_000;
    elsif(enb_096 = '1') then reg_000_096 <= Relu_000;
    elsif(enb_097 = '1') then reg_000_097 <= Relu_000;
    elsif(enb_098 = '1') then reg_000_098 <= Relu_000;
    elsif(enb_099 = '1') then reg_000_099 <= Relu_000;
    elsif(enb_100 = '1') then reg_000_100 <= Relu_000;
    elsif(enb_101 = '1') then reg_000_101 <= Relu_000;
    elsif(enb_102 = '1') then reg_000_102 <= Relu_000;
    elsif(enb_103 = '1') then reg_000_103 <= Relu_000;
    elsif(enb_104 = '1') then reg_000_104 <= Relu_000;
    elsif(enb_105 = '1') then reg_000_105 <= Relu_000;
    elsif(enb_106 = '1') then reg_000_106 <= Relu_000;
    elsif(enb_107 = '1') then reg_000_107 <= Relu_000;
    elsif(enb_108 = '1') then reg_000_108 <= Relu_000;
    elsif(enb_109 = '1') then reg_000_109 <= Relu_000;
    elsif(enb_110 = '1') then reg_000_110 <= Relu_000;
    elsif(enb_111 = '1') then reg_000_111 <= Relu_000;
    elsif(enb_112 = '1') then reg_000_112 <= Relu_000;
    elsif(enb_113 = '1') then reg_000_113 <= Relu_000;
    elsif(enb_114 = '1') then reg_000_114 <= Relu_000;
    elsif(enb_115 = '1') then reg_000_115 <= Relu_000;
    elsif(enb_116 = '1') then reg_000_116 <= Relu_000;
    elsif(enb_117 = '1') then reg_000_117 <= Relu_000;
    elsif(enb_118 = '1') then reg_000_118 <= Relu_000;
    elsif(enb_119 = '1') then reg_000_119 <= Relu_000;
    elsif(enb_120 = '1') then reg_000_120 <= Relu_000;
    elsif(enb_121 = '1') then reg_000_121 <= Relu_000;
    elsif(enb_122 = '1') then reg_000_122 <= Relu_000;
    elsif(enb_123 = '1') then reg_000_123 <= Relu_000;
    elsif(enb_124 = '1') then reg_000_124 <= Relu_000;
    elsif(enb_125 = '1') then reg_000_125 <= Relu_000;
    elsif(enb_126 = '1') then reg_000_126 <= Relu_000;
    elsif(enb_127 = '1') then reg_000_127 <= Relu_000;
    elsif(enb_128 = '1') then reg_000_128 <= Relu_000;
    elsif(enb_129 = '1') then reg_000_129 <= Relu_000;
    elsif(enb_130 = '1') then reg_000_130 <= Relu_000;
    elsif(enb_131 = '1') then reg_000_131 <= Relu_000;
    elsif(enb_132 = '1') then reg_000_132 <= Relu_000;
    elsif(enb_133 = '1') then reg_000_133 <= Relu_000;
    elsif(enb_134 = '1') then reg_000_134 <= Relu_000;
    elsif(enb_135 = '1') then reg_000_135 <= Relu_000;
    elsif(enb_136 = '1') then reg_000_136 <= Relu_000;
    elsif(enb_137 = '1') then reg_000_137 <= Relu_000;
    elsif(enb_138 = '1') then reg_000_138 <= Relu_000;
    elsif(enb_139 = '1') then reg_000_139 <= Relu_000;
    elsif(enb_140 = '1') then reg_000_140 <= Relu_000;
    elsif(enb_141 = '1') then reg_000_141 <= Relu_000;
    elsif(enb_142 = '1') then reg_000_142 <= Relu_000;
    elsif(enb_143 = '1') then reg_000_143 <= Relu_000;
    elsif(enb_144 = '1') then reg_000_144 <= Relu_000;
    elsif(enb_145 = '1') then reg_000_145 <= Relu_000;
    elsif(enb_146 = '1') then reg_000_146 <= Relu_000;
    elsif(enb_147 = '1') then reg_000_147 <= Relu_000;
    elsif(enb_148 = '1') then reg_000_148 <= Relu_000;
    elsif(enb_149 = '1') then reg_000_149 <= Relu_000;
    elsif(enb_150 = '1') then reg_000_150 <= Relu_000;
    elsif(enb_151 = '1') then reg_000_151 <= Relu_000;
    elsif(enb_152 = '1') then reg_000_152 <= Relu_000;
    elsif(enb_153 = '1') then reg_000_153 <= Relu_000;
    elsif(enb_154 = '1') then reg_000_154 <= Relu_000;
    elsif(enb_155 = '1') then reg_000_155 <= Relu_000;
    elsif(enb_156 = '1') then reg_000_156 <= Relu_000;
    elsif(enb_157 = '1') then reg_000_157 <= Relu_000;
    elsif(enb_158 = '1') then reg_000_158 <= Relu_000;
    elsif(enb_159 = '1') then reg_000_159 <= Relu_000;
    elsif(enb_160 = '1') then reg_000_160 <= Relu_000;
    elsif(enb_161 = '1') then reg_000_161 <= Relu_000;
    elsif(enb_162 = '1') then reg_000_162 <= Relu_000;
    elsif(enb_163 = '1') then reg_000_163 <= Relu_000;
    elsif(enb_164 = '1') then reg_000_164 <= Relu_000;
    elsif(enb_165 = '1') then reg_000_165 <= Relu_000;
    elsif(enb_166 = '1') then reg_000_166 <= Relu_000;
    elsif(enb_167 = '1') then reg_000_167 <= Relu_000;
    elsif(enb_168 = '1') then reg_000_168 <= Relu_000;
    elsif(enb_169 = '1') then reg_000_169 <= Relu_000;
    elsif(enb_170 = '1') then reg_000_170 <= Relu_000;
    elsif(enb_171 = '1') then reg_000_171 <= Relu_000;
    elsif(enb_172 = '1') then reg_000_172 <= Relu_000;
    elsif(enb_173 = '1') then reg_000_173 <= Relu_000;
    elsif(enb_174 = '1') then reg_000_174 <= Relu_000;
    elsif(enb_175 = '1') then reg_000_175 <= Relu_000;
    elsif(enb_176 = '1') then reg_000_176 <= Relu_000;
    elsif(enb_177 = '1') then reg_000_177 <= Relu_000;
    elsif(enb_178 = '1') then reg_000_178 <= Relu_000;
    elsif(enb_179 = '1') then reg_000_179 <= Relu_000;
    elsif(enb_180 = '1') then reg_000_180 <= Relu_000;
    elsif(enb_181 = '1') then reg_000_181 <= Relu_000;
    elsif(enb_182 = '1') then reg_000_182 <= Relu_000;
    elsif(enb_183 = '1') then reg_000_183 <= Relu_000;
    elsif(enb_184 = '1') then reg_000_184 <= Relu_000;
    elsif(enb_185 = '1') then reg_000_185 <= Relu_000;
    elsif(enb_186 = '1') then reg_000_186 <= Relu_000;
    elsif(enb_187 = '1') then reg_000_187 <= Relu_000;
    elsif(enb_188 = '1') then reg_000_188 <= Relu_000;
    elsif(enb_189 = '1') then reg_000_189 <= Relu_000;
    elsif(enb_190 = '1') then reg_000_190 <= Relu_000;
    elsif(enb_191 = '1') then reg_000_191 <= Relu_000;
    elsif(enb_192 = '1') then reg_000_192 <= Relu_000;
    elsif(enb_193 = '1') then reg_000_193 <= Relu_000;
    elsif(enb_194 = '1') then reg_000_194 <= Relu_000;
    elsif(enb_195 = '1') then reg_000_195 <= Relu_000;
    elsif(enb_196 = '1') then reg_000_196 <= Relu_000;
    elsif(enb_197 = '1') then reg_000_197 <= Relu_000;
    elsif(enb_198 = '1') then reg_000_198 <= Relu_000;
    elsif(enb_199 = '1') then reg_000_199 <= Relu_000;
    elsif(enb_200 = '1') then reg_000_200 <= Relu_000;
    elsif(enb_201 = '1') then reg_000_201 <= Relu_000;
    elsif(enb_202 = '1') then reg_000_202 <= Relu_000;
    elsif(enb_203 = '1') then reg_000_203 <= Relu_000;
    elsif(enb_204 = '1') then reg_000_204 <= Relu_000;
    elsif(enb_205 = '1') then reg_000_205 <= Relu_000;
    elsif(enb_206 = '1') then reg_000_206 <= Relu_000;
    elsif(enb_207 = '1') then reg_000_207 <= Relu_000;
    elsif(enb_208 = '1') then reg_000_208 <= Relu_000;
    elsif(enb_209 = '1') then reg_000_209 <= Relu_000;
    elsif(enb_210 = '1') then reg_000_210 <= Relu_000;
    elsif(enb_211 = '1') then reg_000_211 <= Relu_000;
    elsif(enb_212 = '1') then reg_000_212 <= Relu_000;
    elsif(enb_213 = '1') then reg_000_213 <= Relu_000;
    elsif(enb_214 = '1') then reg_000_214 <= Relu_000;
    elsif(enb_215 = '1') then reg_000_215 <= Relu_000;
    elsif(enb_216 = '1') then reg_000_216 <= Relu_000;
    elsif(enb_217 = '1') then reg_000_217 <= Relu_000;
    elsif(enb_218 = '1') then reg_000_218 <= Relu_000;
    elsif(enb_219 = '1') then reg_000_219 <= Relu_000;
    elsif(enb_220 = '1') then reg_000_220 <= Relu_000;
    elsif(enb_221 = '1') then reg_000_221 <= Relu_000;
    elsif(enb_222 = '1') then reg_000_222 <= Relu_000;
    elsif(enb_223 = '1') then reg_000_223 <= Relu_000;
    elsif(enb_224 = '1') then reg_000_224 <= Relu_000;
    elsif(enb_225 = '1') then reg_000_225 <= Relu_000;
    elsif(enb_226 = '1') then reg_000_226 <= Relu_000;
    elsif(enb_227 = '1') then reg_000_227 <= Relu_000;
    elsif(enb_228 = '1') then reg_000_228 <= Relu_000;
    elsif(enb_229 = '1') then reg_000_229 <= Relu_000;
    elsif(enb_230 = '1') then reg_000_230 <= Relu_000;
    elsif(enb_231 = '1') then reg_000_231 <= Relu_000;
    elsif(enb_232 = '1') then reg_000_232 <= Relu_000;
    elsif(enb_233 = '1') then reg_000_233 <= Relu_000;
    elsif(enb_234 = '1') then reg_000_234 <= Relu_000;
    elsif(enb_235 = '1') then reg_000_235 <= Relu_000;
    elsif(enb_236 = '1') then reg_000_236 <= Relu_000;
    elsif(enb_237 = '1') then reg_000_237 <= Relu_000;
    elsif(enb_238 = '1') then reg_000_238 <= Relu_000;
    elsif(enb_239 = '1') then reg_000_239 <= Relu_000;
    elsif(enb_240 = '1') then reg_000_240 <= Relu_000;
    elsif(enb_241 = '1') then reg_000_241 <= Relu_000;
    elsif(enb_242 = '1') then reg_000_242 <= Relu_000;
    elsif(enb_243 = '1') then reg_000_243 <= Relu_000;
    elsif(enb_244 = '1') then reg_000_244 <= Relu_000;
    elsif(enb_245 = '1') then reg_000_245 <= Relu_000;
    elsif(enb_246 = '1') then reg_000_246 <= Relu_000;
    elsif(enb_247 = '1') then reg_000_247 <= Relu_000;
    elsif(enb_248 = '1') then reg_000_248 <= Relu_000;
    elsif(enb_249 = '1') then reg_000_249 <= Relu_000;
    elsif(enb_250 = '1') then reg_000_250 <= Relu_000;
    elsif(enb_251 = '1') then reg_000_251 <= Relu_000;
    elsif(enb_252 = '1') then reg_000_252 <= Relu_000;
    elsif(enb_253 = '1') then reg_000_253 <= Relu_000;
    elsif(enb_254 = '1') then reg_000_254 <= Relu_000;
    elsif(enb_255 = '1') then reg_000_255 <= Relu_000;
    end if;
  end if;  
end process;


-------------------------------------------------------
ROM_EW8_inst : ROM_EW8 port map (
  CLK => CLK,
  ADDR => EW8Addr,
  DATA => EW8Data );

---------------------------------------------------------
coeff_000 <= 
EW8Data( 511 downto 510 ) when EW8Sel = "00000000" else 
EW8Data( 509 downto 508 ) when EW8Sel = "00000001" else 
EW8Data( 507 downto 506 ) when EW8Sel = "00000010" else 
EW8Data( 505 downto 504 ) when EW8Sel = "00000011" else 
EW8Data( 503 downto 502 ) when EW8Sel = "00000100" else 
EW8Data( 501 downto 500 ) when EW8Sel = "00000101" else 
EW8Data( 499 downto 498 ) when EW8Sel = "00000110" else 
EW8Data( 497 downto 496 ) when EW8Sel = "00000111" else 
EW8Data( 495 downto 494 ) when EW8Sel = "00001000" else 
EW8Data( 493 downto 492 ) when EW8Sel = "00001001" else 
EW8Data( 491 downto 490 ) when EW8Sel = "00001010" else 
EW8Data( 489 downto 488 ) when EW8Sel = "00001011" else 
EW8Data( 487 downto 486 ) when EW8Sel = "00001100" else 
EW8Data( 485 downto 484 ) when EW8Sel = "00001101" else 
EW8Data( 483 downto 482 ) when EW8Sel = "00001110" else 
EW8Data( 481 downto 480 ) when EW8Sel = "00001111" else 
EW8Data( 479 downto 478 ) when EW8Sel = "00010000" else 
EW8Data( 477 downto 476 ) when EW8Sel = "00010001" else 
EW8Data( 475 downto 474 ) when EW8Sel = "00010010" else 
EW8Data( 473 downto 472 ) when EW8Sel = "00010011" else 
EW8Data( 471 downto 470 ) when EW8Sel = "00010100" else 
EW8Data( 469 downto 468 ) when EW8Sel = "00010101" else 
EW8Data( 467 downto 466 ) when EW8Sel = "00010110" else 
EW8Data( 465 downto 464 ) when EW8Sel = "00010111" else 
EW8Data( 463 downto 462 ) when EW8Sel = "00011000" else 
EW8Data( 461 downto 460 ) when EW8Sel = "00011001" else 
EW8Data( 459 downto 458 ) when EW8Sel = "00011010" else 
EW8Data( 457 downto 456 ) when EW8Sel = "00011011" else 
EW8Data( 455 downto 454 ) when EW8Sel = "00011100" else 
EW8Data( 453 downto 452 ) when EW8Sel = "00011101" else 
EW8Data( 451 downto 450 ) when EW8Sel = "00011110" else 
EW8Data( 449 downto 448 ) when EW8Sel = "00011111" else 
EW8Data( 447 downto 446 ) when EW8Sel = "00100000" else 
EW8Data( 445 downto 444 ) when EW8Sel = "00100001" else 
EW8Data( 443 downto 442 ) when EW8Sel = "00100010" else 
EW8Data( 441 downto 440 ) when EW8Sel = "00100011" else 
EW8Data( 439 downto 438 ) when EW8Sel = "00100100" else 
EW8Data( 437 downto 436 ) when EW8Sel = "00100101" else 
EW8Data( 435 downto 434 ) when EW8Sel = "00100110" else 
EW8Data( 433 downto 432 ) when EW8Sel = "00100111" else 
EW8Data( 431 downto 430 ) when EW8Sel = "00101000" else 
EW8Data( 429 downto 428 ) when EW8Sel = "00101001" else 
EW8Data( 427 downto 426 ) when EW8Sel = "00101010" else 
EW8Data( 425 downto 424 ) when EW8Sel = "00101011" else 
EW8Data( 423 downto 422 ) when EW8Sel = "00101100" else 
EW8Data( 421 downto 420 ) when EW8Sel = "00101101" else 
EW8Data( 419 downto 418 ) when EW8Sel = "00101110" else 
EW8Data( 417 downto 416 ) when EW8Sel = "00101111" else 
EW8Data( 415 downto 414 ) when EW8Sel = "00110000" else 
EW8Data( 413 downto 412 ) when EW8Sel = "00110001" else 
EW8Data( 411 downto 410 ) when EW8Sel = "00110010" else 
EW8Data( 409 downto 408 ) when EW8Sel = "00110011" else 
EW8Data( 407 downto 406 ) when EW8Sel = "00110100" else 
EW8Data( 405 downto 404 ) when EW8Sel = "00110101" else 
EW8Data( 403 downto 402 ) when EW8Sel = "00110110" else 
EW8Data( 401 downto 400 ) when EW8Sel = "00110111" else 
EW8Data( 399 downto 398 ) when EW8Sel = "00111000" else 
EW8Data( 397 downto 396 ) when EW8Sel = "00111001" else 
EW8Data( 395 downto 394 ) when EW8Sel = "00111010" else 
EW8Data( 393 downto 392 ) when EW8Sel = "00111011" else 
EW8Data( 391 downto 390 ) when EW8Sel = "00111100" else 
EW8Data( 389 downto 388 ) when EW8Sel = "00111101" else 
EW8Data( 387 downto 386 ) when EW8Sel = "00111110" else 
EW8Data( 385 downto 384 ) when EW8Sel = "00111111" else 
EW8Data( 383 downto 382 ) when EW8Sel = "01000000" else 
EW8Data( 381 downto 380 ) when EW8Sel = "01000001" else 
EW8Data( 379 downto 378 ) when EW8Sel = "01000010" else 
EW8Data( 377 downto 376 ) when EW8Sel = "01000011" else 
EW8Data( 375 downto 374 ) when EW8Sel = "01000100" else 
EW8Data( 373 downto 372 ) when EW8Sel = "01000101" else 
EW8Data( 371 downto 370 ) when EW8Sel = "01000110" else 
EW8Data( 369 downto 368 ) when EW8Sel = "01000111" else 
EW8Data( 367 downto 366 ) when EW8Sel = "01001000" else 
EW8Data( 365 downto 364 ) when EW8Sel = "01001001" else 
EW8Data( 363 downto 362 ) when EW8Sel = "01001010" else 
EW8Data( 361 downto 360 ) when EW8Sel = "01001011" else 
EW8Data( 359 downto 358 ) when EW8Sel = "01001100" else 
EW8Data( 357 downto 356 ) when EW8Sel = "01001101" else 
EW8Data( 355 downto 354 ) when EW8Sel = "01001110" else 
EW8Data( 353 downto 352 ) when EW8Sel = "01001111" else 
EW8Data( 351 downto 350 ) when EW8Sel = "01010000" else 
EW8Data( 349 downto 348 ) when EW8Sel = "01010001" else 
EW8Data( 347 downto 346 ) when EW8Sel = "01010010" else 
EW8Data( 345 downto 344 ) when EW8Sel = "01010011" else 
EW8Data( 343 downto 342 ) when EW8Sel = "01010100" else 
EW8Data( 341 downto 340 ) when EW8Sel = "01010101" else 
EW8Data( 339 downto 338 ) when EW8Sel = "01010110" else 
EW8Data( 337 downto 336 ) when EW8Sel = "01010111" else 
EW8Data( 335 downto 334 ) when EW8Sel = "01011000" else 
EW8Data( 333 downto 332 ) when EW8Sel = "01011001" else 
EW8Data( 331 downto 330 ) when EW8Sel = "01011010" else 
EW8Data( 329 downto 328 ) when EW8Sel = "01011011" else 
EW8Data( 327 downto 326 ) when EW8Sel = "01011100" else 
EW8Data( 325 downto 324 ) when EW8Sel = "01011101" else 
EW8Data( 323 downto 322 ) when EW8Sel = "01011110" else 
EW8Data( 321 downto 320 ) when EW8Sel = "01011111" else 
EW8Data( 319 downto 318 ) when EW8Sel = "01100000" else 
EW8Data( 317 downto 316 ) when EW8Sel = "01100001" else 
EW8Data( 315 downto 314 ) when EW8Sel = "01100010" else 
EW8Data( 313 downto 312 ) when EW8Sel = "01100011" else 
EW8Data( 311 downto 310 ) when EW8Sel = "01100100" else 
EW8Data( 309 downto 308 ) when EW8Sel = "01100101" else 
EW8Data( 307 downto 306 ) when EW8Sel = "01100110" else 
EW8Data( 305 downto 304 ) when EW8Sel = "01100111" else 
EW8Data( 303 downto 302 ) when EW8Sel = "01101000" else 
EW8Data( 301 downto 300 ) when EW8Sel = "01101001" else 
EW8Data( 299 downto 298 ) when EW8Sel = "01101010" else 
EW8Data( 297 downto 296 ) when EW8Sel = "01101011" else 
EW8Data( 295 downto 294 ) when EW8Sel = "01101100" else 
EW8Data( 293 downto 292 ) when EW8Sel = "01101101" else 
EW8Data( 291 downto 290 ) when EW8Sel = "01101110" else 
EW8Data( 289 downto 288 ) when EW8Sel = "01101111" else 
EW8Data( 287 downto 286 ) when EW8Sel = "01110000" else 
EW8Data( 285 downto 284 ) when EW8Sel = "01110001" else 
EW8Data( 283 downto 282 ) when EW8Sel = "01110010" else 
EW8Data( 281 downto 280 ) when EW8Sel = "01110011" else 
EW8Data( 279 downto 278 ) when EW8Sel = "01110100" else 
EW8Data( 277 downto 276 ) when EW8Sel = "01110101" else 
EW8Data( 275 downto 274 ) when EW8Sel = "01110110" else 
EW8Data( 273 downto 272 ) when EW8Sel = "01110111" else 
EW8Data( 271 downto 270 ) when EW8Sel = "01111000" else 
EW8Data( 269 downto 268 ) when EW8Sel = "01111001" else 
EW8Data( 267 downto 266 ) when EW8Sel = "01111010" else 
EW8Data( 265 downto 264 ) when EW8Sel = "01111011" else 
EW8Data( 263 downto 262 ) when EW8Sel = "01111100" else 
EW8Data( 261 downto 260 ) when EW8Sel = "01111101" else 
EW8Data( 259 downto 258 ) when EW8Sel = "01111110" else 
EW8Data( 257 downto 256 ) when EW8Sel = "01111111" else 
EW8Data( 255 downto 254 ) when EW8Sel = "10000000" else 
EW8Data( 253 downto 252 ) when EW8Sel = "10000001" else 
EW8Data( 251 downto 250 ) when EW8Sel = "10000010" else 
EW8Data( 249 downto 248 ) when EW8Sel = "10000011" else 
EW8Data( 247 downto 246 ) when EW8Sel = "10000100" else 
EW8Data( 245 downto 244 ) when EW8Sel = "10000101" else 
EW8Data( 243 downto 242 ) when EW8Sel = "10000110" else 
EW8Data( 241 downto 240 ) when EW8Sel = "10000111" else 
EW8Data( 239 downto 238 ) when EW8Sel = "10001000" else 
EW8Data( 237 downto 236 ) when EW8Sel = "10001001" else 
EW8Data( 235 downto 234 ) when EW8Sel = "10001010" else 
EW8Data( 233 downto 232 ) when EW8Sel = "10001011" else 
EW8Data( 231 downto 230 ) when EW8Sel = "10001100" else 
EW8Data( 229 downto 228 ) when EW8Sel = "10001101" else 
EW8Data( 227 downto 226 ) when EW8Sel = "10001110" else 
EW8Data( 225 downto 224 ) when EW8Sel = "10001111" else 
EW8Data( 223 downto 222 ) when EW8Sel = "10010000" else 
EW8Data( 221 downto 220 ) when EW8Sel = "10010001" else 
EW8Data( 219 downto 218 ) when EW8Sel = "10010010" else 
EW8Data( 217 downto 216 ) when EW8Sel = "10010011" else 
EW8Data( 215 downto 214 ) when EW8Sel = "10010100" else 
EW8Data( 213 downto 212 ) when EW8Sel = "10010101" else 
EW8Data( 211 downto 210 ) when EW8Sel = "10010110" else 
EW8Data( 209 downto 208 ) when EW8Sel = "10010111" else 
EW8Data( 207 downto 206 ) when EW8Sel = "10011000" else 
EW8Data( 205 downto 204 ) when EW8Sel = "10011001" else 
EW8Data( 203 downto 202 ) when EW8Sel = "10011010" else 
EW8Data( 201 downto 200 ) when EW8Sel = "10011011" else 
EW8Data( 199 downto 198 ) when EW8Sel = "10011100" else 
EW8Data( 197 downto 196 ) when EW8Sel = "10011101" else 
EW8Data( 195 downto 194 ) when EW8Sel = "10011110" else 
EW8Data( 193 downto 192 ) when EW8Sel = "10011111" else 
EW8Data( 191 downto 190 ) when EW8Sel = "10100000" else 
EW8Data( 189 downto 188 ) when EW8Sel = "10100001" else 
EW8Data( 187 downto 186 ) when EW8Sel = "10100010" else 
EW8Data( 185 downto 184 ) when EW8Sel = "10100011" else 
EW8Data( 183 downto 182 ) when EW8Sel = "10100100" else 
EW8Data( 181 downto 180 ) when EW8Sel = "10100101" else 
EW8Data( 179 downto 178 ) when EW8Sel = "10100110" else 
EW8Data( 177 downto 176 ) when EW8Sel = "10100111" else 
EW8Data( 175 downto 174 ) when EW8Sel = "10101000" else 
EW8Data( 173 downto 172 ) when EW8Sel = "10101001" else 
EW8Data( 171 downto 170 ) when EW8Sel = "10101010" else 
EW8Data( 169 downto 168 ) when EW8Sel = "10101011" else 
EW8Data( 167 downto 166 ) when EW8Sel = "10101100" else 
EW8Data( 165 downto 164 ) when EW8Sel = "10101101" else 
EW8Data( 163 downto 162 ) when EW8Sel = "10101110" else 
EW8Data( 161 downto 160 ) when EW8Sel = "10101111" else 
EW8Data( 159 downto 158 ) when EW8Sel = "10110000" else 
EW8Data( 157 downto 156 ) when EW8Sel = "10110001" else 
EW8Data( 155 downto 154 ) when EW8Sel = "10110010" else 
EW8Data( 153 downto 152 ) when EW8Sel = "10110011" else 
EW8Data( 151 downto 150 ) when EW8Sel = "10110100" else 
EW8Data( 149 downto 148 ) when EW8Sel = "10110101" else 
EW8Data( 147 downto 146 ) when EW8Sel = "10110110" else 
EW8Data( 145 downto 144 ) when EW8Sel = "10110111" else 
EW8Data( 143 downto 142 ) when EW8Sel = "10111000" else 
EW8Data( 141 downto 140 ) when EW8Sel = "10111001" else 
EW8Data( 139 downto 138 ) when EW8Sel = "10111010" else 
EW8Data( 137 downto 136 ) when EW8Sel = "10111011" else 
EW8Data( 135 downto 134 ) when EW8Sel = "10111100" else 
EW8Data( 133 downto 132 ) when EW8Sel = "10111101" else 
EW8Data( 131 downto 130 ) when EW8Sel = "10111110" else 
EW8Data( 129 downto 128 ) when EW8Sel = "10111111" else 
EW8Data( 127 downto 126 ) when EW8Sel = "11000000" else 
EW8Data( 125 downto 124 ) when EW8Sel = "11000001" else 
EW8Data( 123 downto 122 ) when EW8Sel = "11000010" else 
EW8Data( 121 downto 120 ) when EW8Sel = "11000011" else 
EW8Data( 119 downto 118 ) when EW8Sel = "11000100" else 
EW8Data( 117 downto 116 ) when EW8Sel = "11000101" else 
EW8Data( 115 downto 114 ) when EW8Sel = "11000110" else 
EW8Data( 113 downto 112 ) when EW8Sel = "11000111" else 
EW8Data( 111 downto 110 ) when EW8Sel = "11001000" else 
EW8Data( 109 downto 108 ) when EW8Sel = "11001001" else 
EW8Data( 107 downto 106 ) when EW8Sel = "11001010" else 
EW8Data( 105 downto 104 ) when EW8Sel = "11001011" else 
EW8Data( 103 downto 102 ) when EW8Sel = "11001100" else 
EW8Data( 101 downto 100 ) when EW8Sel = "11001101" else 
EW8Data( 99 downto 98 ) when EW8Sel = "11001110" else 
EW8Data( 97 downto 96 ) when EW8Sel = "11001111" else 
EW8Data( 95 downto 94 ) when EW8Sel = "11010000" else 
EW8Data( 93 downto 92 ) when EW8Sel = "11010001" else 
EW8Data( 91 downto 90 ) when EW8Sel = "11010010" else 
EW8Data( 89 downto 88 ) when EW8Sel = "11010011" else 
EW8Data( 87 downto 86 ) when EW8Sel = "11010100" else 
EW8Data( 85 downto 84 ) when EW8Sel = "11010101" else 
EW8Data( 83 downto 82 ) when EW8Sel = "11010110" else 
EW8Data( 81 downto 80 ) when EW8Sel = "11010111" else 
EW8Data( 79 downto 78 ) when EW8Sel = "11011000" else 
EW8Data( 77 downto 76 ) when EW8Sel = "11011001" else 
EW8Data( 75 downto 74 ) when EW8Sel = "11011010" else 
EW8Data( 73 downto 72 ) when EW8Sel = "11011011" else 
EW8Data( 71 downto 70 ) when EW8Sel = "11011100" else 
EW8Data( 69 downto 68 ) when EW8Sel = "11011101" else 
EW8Data( 67 downto 66 ) when EW8Sel = "11011110" else 
EW8Data( 65 downto 64 ) when EW8Sel = "11011111" else 
EW8Data( 63 downto 62 ) when EW8Sel = "11100000" else 
EW8Data( 61 downto 60 ) when EW8Sel = "11100001" else 
EW8Data( 59 downto 58 ) when EW8Sel = "11100010" else 
EW8Data( 57 downto 56 ) when EW8Sel = "11100011" else 
EW8Data( 55 downto 54 ) when EW8Sel = "11100100" else 
EW8Data( 53 downto 52 ) when EW8Sel = "11100101" else 
EW8Data( 51 downto 50 ) when EW8Sel = "11100110" else 
EW8Data( 49 downto 48 ) when EW8Sel = "11100111" else 
EW8Data( 47 downto 46 ) when EW8Sel = "11101000" else 
EW8Data( 45 downto 44 ) when EW8Sel = "11101001" else 
EW8Data( 43 downto 42 ) when EW8Sel = "11101010" else 
EW8Data( 41 downto 40 ) when EW8Sel = "11101011" else 
EW8Data( 39 downto 38 ) when EW8Sel = "11101100" else 
EW8Data( 37 downto 36 ) when EW8Sel = "11101101" else 
EW8Data( 35 downto 34 ) when EW8Sel = "11101110" else 
EW8Data( 33 downto 32 ) when EW8Sel = "11101111" else 
EW8Data( 31 downto 30 ) when EW8Sel = "11110000" else 
EW8Data( 29 downto 28 ) when EW8Sel = "11110001" else 
EW8Data( 27 downto 26 ) when EW8Sel = "11110010" else 
EW8Data( 25 downto 24 ) when EW8Sel = "11110011" else 
EW8Data( 23 downto 22 ) when EW8Sel = "11110100" else 
EW8Data( 21 downto 20 ) when EW8Sel = "11110101" else 
EW8Data( 19 downto 18 ) when EW8Sel = "11110110" else 
EW8Data( 17 downto 16 ) when EW8Sel = "11110111" else 
EW8Data( 15 downto 14 ) when EW8Sel = "11111000" else 
EW8Data( 13 downto 12 ) when EW8Sel = "11111001" else 
EW8Data( 11 downto 10 ) when EW8Sel = "11111010" else 
EW8Data( 9 downto 8 ) when EW8Sel = "11111011" else 
EW8Data( 7 downto 6 ) when EW8Sel = "11111100" else 
EW8Data( 5 downto 4 ) when EW8Sel = "11111101" else 
EW8Data( 3 downto 2 ) when EW8Sel = "11111110" else 
EW8Data( 1 downto 0 );--- when EW8Sel = "11111111" else 


---------------------------------------------------------
dotpro_ex8_000 : dotpro_ex8 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_000,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_000 );

-------------------------------------------------------
  truncate_000 <= mac_000(15 downto 6);

---------------------------------------------------------
  Relu_000 <= (others => '0') when truncate_000 < 0 else
             (others => '1') when truncate_000 > 255 else
             truncate_000(7 downto 0);


--ReluSum <= ("0000000" & Relu_00) + ("0000000" & Relu_01)
--		+ ("0000000" & Relu_02) + ("0000000" & Relu_03)
--		+ ("0000000" & Relu_04) + ("0000000" & Relu_05)
--		+ ("0000000" & Relu_06) + ("0000000" & Relu_07)
--		+ ("0000000" & Relu_08) + ("0000000" & Relu_09)
--
--		+ ("0000000" & Relu_10) + ("0000000" & Relu_11)
--		+ ("0000000" & Relu_12) + ("0000000" & Relu_13)
--		+ ("0000000" & Relu_14) + ("0000000" & Relu_15)
--		+ ("0000000" & Relu_16) + ("0000000" & Relu_17)
--		+ ("0000000" & Relu_18) + ("0000000" & Relu_19)
--
--		+ ("0000000" & Relu_20) + ("0000000" & Relu_21)
--		+ ("0000000" & Relu_22) + ("0000000" & Relu_23)
--		+ ("0000000" & Relu_24) + ("0000000" & Relu_25)
--		+ ("0000000" & Relu_26) + ("0000000" & Relu_27)
--		+ ("0000000" & Relu_28) + ("0000000" & Relu_29)
--
--		+ ("0000000" & Relu_30) + ("0000000" & Relu_31);

ReluAll <= 
	reg_000_000 & reg_000_001 & reg_000_002 & reg_000_003 & reg_000_004 & reg_000_005 & reg_000_006 & reg_000_007 & reg_000_008 & reg_000_009 & 
	reg_000_010 & reg_000_011 & reg_000_012 & reg_000_013 & reg_000_014 & reg_000_015 & reg_000_016 & reg_000_017 & reg_000_018 & reg_000_019 & 
	reg_000_020 & reg_000_021 & reg_000_022 & reg_000_023 & reg_000_024 & reg_000_025 & reg_000_026 & reg_000_027 & reg_000_028 & reg_000_029 & 
	reg_000_030 & reg_000_031 & reg_000_032 & reg_000_033 & reg_000_034 & reg_000_035 & reg_000_036 & reg_000_037 & reg_000_038 & reg_000_039 & 
	reg_000_040 & reg_000_041 & reg_000_042 & reg_000_043 & reg_000_044 & reg_000_045 & reg_000_046 & reg_000_047 & reg_000_048 & reg_000_049 & 
	reg_000_050 & reg_000_051 & reg_000_052 & reg_000_053 & reg_000_054 & reg_000_055 & reg_000_056 & reg_000_057 & reg_000_058 & reg_000_059 & 
	reg_000_060 & reg_000_061 & reg_000_062 & reg_000_063 & reg_000_064 & reg_000_065 & reg_000_066 & reg_000_067 & reg_000_068 & reg_000_069 & 
	reg_000_070 & reg_000_071 & reg_000_072 & reg_000_073 & reg_000_074 & reg_000_075 & reg_000_076 & reg_000_077 & reg_000_078 & reg_000_079 & 
	reg_000_080 & reg_000_081 & reg_000_082 & reg_000_083 & reg_000_084 & reg_000_085 & reg_000_086 & reg_000_087 & reg_000_088 & reg_000_089 & 
	reg_000_090 & reg_000_091 & reg_000_092 & reg_000_093 & reg_000_094 & reg_000_095 & reg_000_096 & reg_000_097 & reg_000_098 & reg_000_099 & 
	reg_000_100 & reg_000_101 & reg_000_102 & reg_000_103 & reg_000_104 & reg_000_105 & reg_000_106 & reg_000_107 & reg_000_108 & reg_000_109 & 
	reg_000_110 & reg_000_111 & reg_000_112 & reg_000_113 & reg_000_114 & reg_000_115 & reg_000_116 & reg_000_117 & reg_000_118 & reg_000_119 & 
	reg_000_120 & reg_000_121 & reg_000_122 & reg_000_123 & reg_000_124 & reg_000_125 & reg_000_126 & reg_000_127 & reg_000_128 & reg_000_129 & 
	reg_000_130 & reg_000_131 & reg_000_132 & reg_000_133 & reg_000_134 & reg_000_135 & reg_000_136 & reg_000_137 & reg_000_138 & reg_000_139 & 
	reg_000_140 & reg_000_141 & reg_000_142 & reg_000_143 & reg_000_144 & reg_000_145 & reg_000_146 & reg_000_147 & reg_000_148 & reg_000_149 & 
	reg_000_150 & reg_000_151 & reg_000_152 & reg_000_153 & reg_000_154 & reg_000_155 & reg_000_156 & reg_000_157 & reg_000_158 & reg_000_159 & 
	reg_000_160 & reg_000_161 & reg_000_162 & reg_000_163 & reg_000_164 & reg_000_165 & reg_000_166 & reg_000_167 & reg_000_168 & reg_000_169 & 
	reg_000_170 & reg_000_171 & reg_000_172 & reg_000_173 & reg_000_174 & reg_000_175 & reg_000_176 & reg_000_177 & reg_000_178 & reg_000_179 & 
	reg_000_180 & reg_000_181 & reg_000_182 & reg_000_183 & reg_000_184 & reg_000_185 & reg_000_186 & reg_000_187 & reg_000_188 & reg_000_189 & 
	reg_000_190 & reg_000_191 & reg_000_192 & reg_000_193 & reg_000_194 & reg_000_195 & reg_000_196 & reg_000_197 & reg_000_198 & reg_000_199 & 
	reg_000_200 & reg_000_201 & reg_000_202 & reg_000_203 & reg_000_204 & reg_000_205 & reg_000_206 & reg_000_207 & reg_000_208 & reg_000_209 & 
	reg_000_210 & reg_000_211 & reg_000_212 & reg_000_213 & reg_000_214 & reg_000_215 & reg_000_216 & reg_000_217 & reg_000_218 & reg_000_219 & 
	reg_000_220 & reg_000_221 & reg_000_222 & reg_000_223 & reg_000_224 & reg_000_225 & reg_000_226 & reg_000_227 & reg_000_228 & reg_000_229 & 
	reg_000_230 & reg_000_231 & reg_000_232 & reg_000_233 & reg_000_234 & reg_000_235 & reg_000_236 & reg_000_237 & reg_000_238 & reg_000_239 & 
	reg_000_240 & reg_000_241 & reg_000_242 & reg_000_243 & reg_000_244 & reg_000_245 & reg_000_246 & reg_000_247 & reg_000_248 & reg_000_249 & 
	reg_000_250 & reg_000_251 & reg_000_252 & reg_000_253 & reg_000_254 & reg_000_255 ; 


ReluAllWrN <= not enb_255; -- 2022/11/04 for ex4, to write the very first data correctly to the RAM_X2

end RTL;

